@@ -27,7 +27,7 @@ from setproctitle import setproctitle
 from ai.backend.client.config import APIConfig
 from ai.backend.client.exceptions import BackendAPIError, BackendClientError
 from ai.backend.client.session import AsyncSession as APISession
-from ai.backend.common.web.session import get_session
+from ai.backend.common.web.session import extra_config_headers, get_session
 from ai.backend.common.web.session import setup as setup_session
 from ai.backend.common.web.session.redis_storage import RedisStorage
 
@@ -186,6 +186,12 @@ async def login_handler(request: web.Request) -> web.Response:
             ),
             content_type="application/problem+json",
         )
+    request_headers = extra_config_headers.check(request.headers)
+    secure_context = request_headers.get("X-BackendAI-Encoded", None)
+    if not secure_context:
+        # For non-encrypted requests, just read the body as-is.
+        # Encrypted requests are handled by the `decrypt_payload` middleware.
+        request["payload"] = await request.text()
     try:
         creds = json.loads(request["payload"])
     except json.JSONDecodeError as e:
@@ -511,9 +517,10 @@ async def server_main(
     cors.add(app.router.add_route("POST", "/server/login-check", login_check_handler))
     cors.add(app.router.add_route("POST", "/server/logout", logout_handler))
     cors.add(app.router.add_route("GET", "/func/ping", webserver_healthcheck))
-    cors.add(app.router.add_route("GET", "/func/{path:hanati/user}", anon_web_plugin_handler))
     cors.add(app.router.add_route("GET", "/func/{path:cloud/.*$}", anon_web_plugin_handler))
     cors.add(app.router.add_route("POST", "/func/{path:cloud/.*$}", anon_web_plugin_handler))
+    cors.add(app.router.add_route("POST", "/func/{path:custom-auth/.*$}", anon_web_plugin_handler))
+    cors.add(app.router.add_route("GET", "/func/{path:custom-auth/.*$}", anon_web_plugin_handler))
     cors.add(app.router.add_route("POST", "/func/{path:saml/.*$}", anon_web_plugin_handler))
     cors.add(app.router.add_route("POST", "/func/{path:auth/signup}", anon_web_plugin_handler))
     cors.add(app.router.add_route("POST", "/func/{path:auth/signout}", web_handler))
@@ -545,6 +552,12 @@ async def server_main(
     app.on_shutdown.append(server_shutdown)
     app.on_cleanup.append(server_cleanup)
 
+    async def on_prepare(request, response):
+        # Remove "Server" header for a security reason.
+        response.headers.popall("Server", None)
+
+    app.on_response_prepare.append(on_prepare)
+
     ssl_ctx = None
     if config["service"]["ssl_enabled"]:
         ssl_ctx = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
